@@ -11,8 +11,6 @@
 #include <ostream>
 #include <string>
 
-#include <libcamera/base/compiler.h>
-
 namespace libcamera {
 
 class Rectangle;
@@ -33,7 +31,7 @@ public:
 	int x;
 	int y;
 
-	const std::string toString() const;
+	std::string toString() const;
 
 	constexpr Point operator-() const
 	{
@@ -66,7 +64,7 @@ public:
 	unsigned int height;
 
 	bool isNull() const { return !width && !height; }
-	const std::string toString() const;
+	std::string toString() const;
 
 	Size &alignDownTo(unsigned int hAlignment, unsigned int vAlignment)
 	{
@@ -110,8 +108,14 @@ public:
 		return *this;
 	}
 
-	__nodiscard constexpr Size alignedDownTo(unsigned int hAlignment,
-						 unsigned int vAlignment) const
+	Size &transpose()
+	{
+		std::swap(width, height);
+		return *this;
+	}
+
+	[[nodiscard]] constexpr Size alignedDownTo(unsigned int hAlignment,
+						   unsigned int vAlignment) const
 	{
 		return {
 			width / hAlignment * hAlignment,
@@ -119,8 +123,8 @@ public:
 		};
 	}
 
-	__nodiscard constexpr Size alignedUpTo(unsigned int hAlignment,
-					       unsigned int vAlignment) const
+	[[nodiscard]] constexpr Size alignedUpTo(unsigned int hAlignment,
+						 unsigned int vAlignment) const
 	{
 		return {
 			(width + hAlignment - 1) / hAlignment * hAlignment,
@@ -128,7 +132,7 @@ public:
 		};
 	}
 
-	__nodiscard constexpr Size boundedTo(const Size &bound) const
+	[[nodiscard]] constexpr Size boundedTo(const Size &bound) const
 	{
 		return {
 			std::min(width, bound.width),
@@ -136,7 +140,7 @@ public:
 		};
 	}
 
-	__nodiscard constexpr Size expandedTo(const Size &expand) const
+	[[nodiscard]] constexpr Size expandedTo(const Size &expand) const
 	{
 		return {
 			std::max(width, expand.width),
@@ -144,7 +148,7 @@ public:
 		};
 	}
 
-	__nodiscard constexpr Size grownBy(const Size &margins) const
+	[[nodiscard]] constexpr Size grownBy(const Size &margins) const
 	{
 		return {
 			width + margins.width,
@@ -152,7 +156,7 @@ public:
 		};
 	}
 
-	__nodiscard constexpr Size shrunkBy(const Size &margins) const
+	[[nodiscard]] constexpr Size shrunkBy(const Size &margins) const
 	{
 		return {
 			width > margins.width ? width - margins.width : 0,
@@ -160,10 +164,10 @@ public:
 		};
 	}
 
-	__nodiscard Size boundedToAspectRatio(const Size &ratio) const;
-	__nodiscard Size expandedToAspectRatio(const Size &ratio) const;
+	[[nodiscard]] Size boundedToAspectRatio(const Size &ratio) const;
+	[[nodiscard]] Size expandedToAspectRatio(const Size &ratio) const;
 
-	__nodiscard Rectangle centeredTo(const Point &center) const;
+	[[nodiscard]] Rectangle centeredTo(const Point &center) const;
 
 	Size operator*(float factor) const;
 	Size operator/(float factor) const;
@@ -277,7 +281,7 @@ public:
 	unsigned int height;
 
 	bool isNull() const { return !width && !height; }
-	const std::string toString() const;
+	std::string toString() const;
 
 	Point center() const;
 
@@ -294,11 +298,11 @@ public:
 	Rectangle &scaleBy(const Size &numerator, const Size &denominator);
 	Rectangle &translateBy(const Point &point);
 
-	__nodiscard Rectangle boundedTo(const Rectangle &bound) const;
-	__nodiscard Rectangle enclosedIn(const Rectangle &boundary) const;
-	__nodiscard Rectangle scaledBy(const Size &numerator,
-				       const Size &denominator) const;
-	__nodiscard Rectangle translatedBy(const Point &point) const;
+	[[nodiscard]] Rectangle boundedTo(const Rectangle &bound) const;
+	[[nodiscard]] Rectangle enclosedIn(const Rectangle &boundary) const;
+	[[nodiscard]] Rectangle scaledBy(const Size &numerator,
+					 const Size &denominator) const;
+	[[nodiscard]] Rectangle translatedBy(const Point &point) const;
 
 	Rectangle transformedBetween(const Rectangle &source,
 				     const Rectangle &target) const;
