@@ -18,6 +18,7 @@
 #include <libcamera/base/thread.h>
 #include <libcamera/base/thread_annotations.h>
 
+#include "libcamera/internal/global_configuration.h"
 #include "libcamera/internal/process.h"
 
 namespace libcamera {
@@ -38,6 +39,11 @@ public:
 	void addCamera(std::shared_ptr<Camera> camera) LIBCAMERA_TSA_EXCLUDES(mutex_);
 	void removeCamera(std::shared_ptr<Camera> camera) LIBCAMERA_TSA_EXCLUDES(mutex_);
 
+	const GlobalConfiguration &configuration() const
+	{
+		return configuration_;
+	}
+
 	IPAManager *ipaManager() const { return ipaManager_.get(); }
 
 protected:
@@ -65,6 +71,8 @@ private:
 	std::unique_ptr<DeviceEnumerator> enumerator_;
 
 	std::unique_ptr<IPAManager> ipaManager_;
+
+	const GlobalConfiguration configuration_;
 };
 
 } /* namespace libcamera */
