@@ -20,6 +20,11 @@ namespace libcamera {
  * wrap around.
  */
 struct SwIspStats {
+	/**
+	 * \brief True if the statistics buffer contains valid data, false if
+	 *        no statistics were generated for this frame
+	 */
+	bool valid;
 	/**
 	 * \brief Holds the sum of all sampled red pixels
 	 */
