@@ -15,7 +15,7 @@ namespace libcamera {
 extern "C" {
 #endif
 
-#define IPA_CONTROLS_FORMAT_VERSION	1
+#define IPA_CONTROLS_FORMAT_VERSION	2
 
 enum ipa_controls_id_map_type {
 	IPA_CONTROL_ID_MAP_CONTROLS,
@@ -34,20 +34,26 @@ struct ipa_controls_header {
 };
 
 struct ipa_control_value_entry {
-	uint32_t id;
 	uint8_t type;
 	uint8_t is_array;
 	uint16_t count;
 	uint32_t offset;
-	uint32_t padding[1];
+	uint32_t reserved[2];
+};
+
+struct ipa_control_list_entry {
+	uint32_t id;
+	struct ipa_control_value_entry value;
 };
 
 struct ipa_control_info_entry {
 	uint32_t id;
 	uint32_t type;
-	uint32_t offset;
 	uint8_t direction;
-	uint8_t padding[3];
+	uint8_t padding[7];
+	struct ipa_control_value_entry min;
+	struct ipa_control_value_entry max;
+	struct ipa_control_value_entry def;
 };
 
 #ifdef __cplusplus
