@@ -2,11 +2,9 @@
 /*
  * Copyright (C) 2021, Collabora Ltd.
  *
- * test_base.h - Base definitions for tests
+ * Base definitions for tests
  */
-
-#ifndef __LC_COMPLIANCE_TEST_BASE_H__
-#define __LC_COMPLIANCE_TEST_BASE_H__
+#pragma once
 
 #include <libcamera/libcamera.h>
 
@@ -20,5 +18,3 @@ protected:
 
 	std::shared_ptr<libcamera::Camera> camera_;
 };
-
-#endif /* __LC_COMPLIANCE_TEST_BASE_H__ */
