@@ -333,6 +333,9 @@ int MainWindow::openCamera()
 
 void MainWindow::toggleCapture(bool start)
 {
+	if (!camera_)
+		return;
+
 	if (start) {
 		startCapture();
 		startStopAction_->setIcon(iconStop_);
@@ -356,6 +359,9 @@ int MainWindow::startCapture()
 
 	/* Verify roles are supported. */
 	switch (roles.size()) {
+	case 0:
+		roles.push_back(StreamRole::Viewfinder);
+		break;
 	case 1:
 		if (roles[0] != StreamRole::Viewfinder) {
 			qWarning() << "Only viewfinder supported for single stream";
