@@ -192,12 +192,13 @@ void gst_libcamera_pad_set_video_info(GstPad *pad, const GstVideoInfo *info)
 Stream *
 gst_libcamera_pad_get_stream(GstPad *pad)
 {
-	auto *self = GST_LIBCAMERA_PAD(pad);
+	return static_cast<Stream *>(gst_pad_get_element_private(pad));
+}
 
-	if (self->pool)
-		return gst_libcamera_pool_get_stream(self->pool);
-
-	return nullptr;
+void
+gst_libcamera_pad_set_stream(GstPad *pad, Stream *stream)
+{
+	gst_pad_set_element_private(pad, stream);
 }
 
 void
