@@ -187,7 +187,8 @@ std::string time_point_to_string(const time_point &time)
 }
 
 std::basic_ostream<char, std::char_traits<char>> &
-operator<<(std::basic_ostream<char, std::char_traits<char>> &stream, const _hex &h)
+details::operator<<(std::basic_ostream<char, std::char_traits<char>> &stream,
+		    const details::hex &h)
 {
 	stream << "0x";
 
@@ -425,6 +426,12 @@ std::string toAscii(const std::string &str)
  * \return The tick count of the Duration expressed in \a Period
  */
 
+/**
+ * \fn Duration::operator-()
+ * \brief Negation operator to negate a \a Duration
+ * \return The duration, with the number of ticks negated
+ */
+
 /**
  * \fn Duration::operator bool()
  * \brief Boolean operator to test if a \a Duration holds a non-zero time value
