@@ -330,7 +330,8 @@ FOV calcFOV(const Size &in, const ImgUDevice::PipeConfig &pipe)
  *
  * \return 0 on success or a negative error code otherwise
  */
-int ImgUDevice::init(MediaDevice *media, unsigned int index)
+int ImgUDevice::init(std::shared_ptr<libcamera::MediaDevice> media,
+		     unsigned int index)
 {
 	int ret;
 
@@ -342,32 +343,32 @@ int ImgUDevice::init(MediaDevice *media, unsigned int index)
 	 * by the match() function: no need to check for newly created
 	 * video devices and subdevice validity here.
 	 */
-	imgu_ = V4L2Subdevice::fromEntityName(media, name_);
+	imgu_ = V4L2Subdevice::fromEntityName(media.get(), name_);
 	ret = imgu_->open();
 	if (ret)
 		return ret;
 
-	input_ = V4L2VideoDevice::fromEntityName(media, name_ + " input");
+	input_ = V4L2VideoDevice::fromEntityName(media.get(), name_ + " input");
 	ret = input_->open();
 	if (ret)
 		return ret;
 
-	output_ = V4L2VideoDevice::fromEntityName(media, name_ + " output");
+	output_ = V4L2VideoDevice::fromEntityName(media.get(), name_ + " output");
 	ret = output_->open();
 	if (ret)
 		return ret;
 
-	viewfinder_ = V4L2VideoDevice::fromEntityName(media, name_ + " viewfinder");
+	viewfinder_ = V4L2VideoDevice::fromEntityName(media.get(), name_ + " viewfinder");
 	ret = viewfinder_->open();
 	if (ret)
 		return ret;
 
-	param_ = V4L2VideoDevice::fromEntityName(media, name_ + " parameters");
+	param_ = V4L2VideoDevice::fromEntityName(media.get(), name_ + " parameters");
 	ret = param_->open();
 	if (ret)
 		return ret;
 
-	stat_ = V4L2VideoDevice::fromEntityName(media, name_ + " 3a stat");
+	stat_ = V4L2VideoDevice::fromEntityName(media.get(), name_ + " 3a stat");
 	ret = stat_->open();
 	if (ret)
 		return ret;
